#include <stdio.h>
#include <poll.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>

#define GPIO_FN_MAXLEN  32
#define RDBUF_LEN       5

int main(int argc, char **argv) {
	char fn[GPIO_FN_MAXLEN];
	int fd[argc-1],ret,i;
	struct pollfd pfd[argc-1];
	char rdbuf[RDBUF_LEN];
	char last[argc-1];

	memset(rdbuf, 0x00, RDBUF_LEN);
	memset(fn, 0x00, GPIO_FN_MAXLEN);

	if(argc<2) {
		printf("Usage: %s <GPIO> ...\nGPIO must be exported to sysfs and have enabled edge detection\n", argv[0]);
		return 1;
	}

	for(i=0; i<argc-1; i++) {
		fd[i]=open("/sys/class/gpio/export", O_WRONLY);
		if(fd[i]<0) {
			perror("export");
		}
		if(fd[i]>0) {
			write(fd[i], argv[i+1], strlen(argv[i+1]));
			close(fd[i]);
		}
		snprintf(fn, GPIO_FN_MAXLEN-1, "/sys/class/gpio/gpio%s/edge", argv[i+1]);
		fd[i]=open(fn, O_WRONLY);
		if(fd[i]<0) {
			perror(fn);
		}
		if(fd[i]>0) {
			write(fd[i], "both", 4);
			close(fd[i]);
		}

		snprintf(fn, GPIO_FN_MAXLEN-1, "/sys/class/gpio/gpio%s/value", argv[i+1]);
		fd[i]=open(fn, O_RDONLY);
		if(fd[i]<0) {
			perror(fn);
			return 2;
		}
		pfd[i].fd=fd[i];
		pfd[i].events=POLLPRI;

		ret=read(fd[i], rdbuf, RDBUF_LEN-1);
		if(ret<0) {
			perror("read()");
			return 4;
		}
		last[i]=rdbuf[0];
	}

	while(1) {
		memset(rdbuf, 0x00, RDBUF_LEN);

		for(i=0; i<argc-1; i++) {
			lseek(fd[i], 0, SEEK_SET);
		}

		ret=poll(pfd, argc-1, -1);
		if(ret<0) {
			perror("poll()");
			close(fd);
			return 3;
		}

		for(i=0; i<argc-1; i++) {
			ret=read(fd[i], rdbuf, RDBUF_LEN-1);
			if(ret<0) {
				perror("read()");
				return 4;
			}
			if (pfd[i].revents & POLLPRI && last[i]!=rdbuf[0]) {
				last[i]=rdbuf[0];
				printf("%s %c\n", argv[i+1], rdbuf[0]);
			}
		}
		fflush(stdout);
	}
}